# This Python file uses the following encoding: utf-8

import codecs
import sys
import getopt
import os
from landmark_ml.learning.PageManager import PageManager

import logging
from landmark_ml.extraction.ExtractionCheck import run_extraction_check
import json
import urllib2
import chardet
from landmark_ml.learning.DivListLearner import DivListLearner
from landmark_extractor.extraction.Landmark import flattenResult
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("landmark")
handler = logging.FileHandler('landmark.log')
handler.setLevel(logging.INFO)
formatter = logging.Formatter(u'%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

write_debug_files = False

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def run(page_file_dir, ignore_files = []):
    test_pages = []
    pageManager = PageManager(write_debug_files)
    
    if os.path.isfile(page_file_dir):
        with open(page_file_dir) as f:
            urls = f.readlines()
        for url in urls:
            page_url = url.strip()
            req = urllib2.urlopen(page_url)
            page_contents = req.read()
            charset = chardet.detect(page_contents)
            page_encoding = charset['encoding']
            page_str = page_contents.decode(page_encoding).encode('utf-8')
            pageManager.addPage(page_url, page_contents)
            test_pages.append(page_url)
    else:
        files = [f for f in os.listdir(page_file_dir) if os.path.isfile(os.path.join(page_file_dir, f))]
        for the_file in files:
            if the_file.startswith('.') or the_file.endswith('.jl') or the_file.endswith('.json') or the_file in ignore_files:
                continue
             
            with codecs.open(os.path.join(page_file_dir, the_file), "r", "utf-8") as myfile:
                page_str = myfile.read().encode('utf-8')
                 
            pageManager.addPage(the_file, page_str)
            test_pages.append(the_file)

    pageManager.learnStripes()
    
    ##table, ul, etc. list learning
#         (list_markup, list_names) = pageManager.learnListMarkups()
#         list_rules = pageManager.learnRulesFromMarkup(list_markup)
    
    ##div learning
#         train_pages = {}
#         for page_id in pageManager._pages:
#             train_pages[page_id] = pageManager.getPage(page_id).getString()
#         d = DivListLearner()
#         div_rules, div_markup = d.run(train_pages)
    
    rule_set = pageManager.learnAllRules()
    rule_set.removeBadRules(test_pages)
    
#         for rule in list_rules.rules:
#             rule_set.add_rule(rule)
#         
#         for rule in div_rules.rules:
#             rule_set.add_rule(rule)

    for page in test_pages:
        print "extract for " + str(page)
        print "------------"
        contents = pageManager.getPage(page).getString()
        extraction_list = rule_set.extract(contents)
        flattened_extraction = flattenResult(extraction_list)
        print json.dumps(flattened_extraction, sort_keys=True, indent=2, separators=(',', ': '))
        
    return rule_set

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "dh", ["debug", "help"])
            
            for opt in opts:
                if opt in [('-d', ''), ('--debug', '')]:
                    write_debug_files = True
                if opt in [('-h', ''), ('--help', '')]:
                    raise Usage('python -m learning.RuleLearnerAllSlots [OPTIONAL_PARAMS] [TEST_FILES_FOLDER] \n\t[OPTIONAL_PARAMS]: -d to get debug stripe html files')
        except getopt.error, msg:
            raise Usage(msg)
        
        logger.info('Running RuleLearnerAllSlots All Slots with files at %s', args[0])
        
        #read the directory location from arg0
        page_file_dir = args[0]
        rule_set = run(page_file_dir)
        
        print json.dumps(json.loads(rule_set.toJson()), sort_keys=True, indent=2, separators=(',', ': '))
        
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2
    
if __name__ == '__main__':
    sys.exit(main())


from landmark_ml.learning.TreeNode import TreeNode
import re

class Tree:

    def __init__(self):
        self.__nodes = {}
        self.__min_occurrences = 5.0
        self.__tree = []
        self.__path_delimeter = "<BRK>"

    def getPathDelimeter(self):
        return self.__path_delimeter

    def getMinOccurrences(self):
        return self.__min_occurrences

    def setMinOccurrences(self, min_occurences):
        self.__min_occurrences = min_occurences

    def add_node_set(self, nodes, meta_data):
        for i in range(len(nodes)):
            depth = i
            parent = None
            if i > 0:
                parent = nodes[i-1]
            path_nodes = nodes[:i]
            path_nodes.reverse()
            path_to_node = self.__path_delimeter.join(path_nodes)
            self.add_node(nodes[i], depth, meta_data, path_to_node=path_to_node, parent=parent)

    def add_node(self, identifier, depth, meta_data, path_to_node=None, parent=None):
        if depth not in self.__nodes:
            self.__nodes[depth] = {}

        if identifier not in self.__nodes[depth]:
            nd = TreeNode(identifier, parent, path_to_node)
            nd.updateMetaData(meta_data)
            self.__nodes[depth][identifier] = [nd]
        else:
            # make sure we have the right one!
            found = None
            for nd_add in self.__nodes[depth][identifier]:
                if nd_add.getPath() == path_to_node:
                    found = nd_add
                    break
            if found is not None:
                found.updateMetaData(meta_data)
            else:
                nd = TreeNode(identifier, parent, path_to_node)
                nd.updateMetaData(meta_data)
                self.__nodes[depth][identifier].append(nd)

        for n in self.__nodes[depth][identifier]:
            if identifier.find("rb_loopadlisting_data") > -1:
                print "== %d %s ==" % (depth, identifier)
                print n.getPath()
                print str(n)

        if parent is not None:
            parent_path = self.__path_delimeter.join(path_to_node.split(self.__path_delimeter)[:-1])
            for parent_node in self.__nodes[depth-1][parent]:
                if parent_node.getPath() == parent_path:
                    parent_node.addChild(identifier)

    def valid_paths(self):  # get the valid paths, along with their visible texts...
        # first, get rid of the nodes that account for very little...
        self.prune()

        # now, find all the leaves. Since each node contains it's own paths, we are fine.
        valid = {}
        for depth in self.__nodes:
            for identifier in self.__nodes[depth]:
                for node in self.__nodes[depth][identifier]:
                    if identifier.find("rb_loopadlisting_data") > -1:
                        print "%d %s" % (depth, identifier)
                        md = node.getMetaData()
                        for n in md:
                            if n['page_id'] == 'tmp1.html':
                                print '>>>> %d %s' % (n['first_vis_token_loc'], n['visible_text'])
                        #print str(node.getMetaData())

                    if len(node.getChildren()) == 0:
                        if len(node.getMetaData()) >= self.__min_occurrences:
                            valid[node.getPath()+self.__path_delimeter+identifier] = node.getMetaData()
        return valid

    def prune(self):
        pruners = []
        # find the possible nodes to prune: those that don't occur enough
        for depth in self.__nodes:
            for identifier in self.__nodes[depth]:
                for node in self.__nodes[depth][identifier]:
                    children = node.getChildren()
                    remove_child = []
                    for child in children:
                        if children[child] < self.__min_occurrences:
                            #print "PRUNING: %s (%d)" % ( node.getPath()+self.__path_delimeter+child, children[child])
                            #print str(node)
                            pruners.append((depth+1, child, node.getPath()+self.__path_delimeter+child))
                            remove_child.append(child)
                    for child in remove_child:
                        del children[child]

        # now we do the pruning (since we can't remove as we iterate above)
        for (depth, id, path) in pruners:
            pot_nodes = self.__nodes[depth][id]
            del_idx = 0
            for i in range(len(pot_nodes)):
                nd = pot_nodes[i]
                if nd.getPath() == path:
                    del_idx = i
                    break
            #print "REMOVE: %d %s %s : %d" % (depth, id, path, del_idx)
            del self.__nodes[depth][id][del_idx]

    def display(self, identifier, depth=0):
        for depth in self.__nodes:
            for identifier in self.__nodes[depth]:
                for nd in self.__nodes[depth][identifier]:
                    print depth*"\t"+str(nd)



    def learn_list_rules(self, markup_by_page, page_mgr):
        from landmark_ml.learning import PageManager

        page_mgr.learnStripes(markups=markup_by_page)
        rules = page_mgr.learnRulesFromMarkup(markup_by_page)

        # now, for each markup rule, learn a little page manager
        sublist_page_managers = {}
        for page in markup_by_page:
            for rule_name in markup_by_page[page]:
                if rule_name not in sublist_page_managers:
                    sublist_page_managers[rule_name] = PageManager()
                for rid in range(len(markup_by_page[page][rule_name]['sequence'])):
                    row = markup_by_page[page][rule_name]['sequence'][rid]
                    sublist_page_managers[rule_name].addPage(page + "html%d" % rid, row['extract'])

        sublist_sub_rules = {}
        for sublist in sublist_page_managers:
            sublist_page_managers[sublist].learnStripes()
            sub_rules = sublist_page_managers[sublist].learnAllRules()
            print '====== SUB RULES ====='
            print sub_rules.toJson()
            for sub_rule in sub_rules.rules:
                sub_rule.removehtml = True
            sublist_sub_rules[sublist] = sub_rules  # This should match a rule name in the rules...

        count = 1
        for rule in rules.rules:
            print "== RULE INFO =="
            print str(rule.name)
            rule.set_sub_rules(sublist_sub_rules[rule.name])
            list_name = '_div_list' + format(count, '04')
            for page_id in markup_by_page:
                if rule.name in markup_by_page[page_id]:
                    markup_by_page[page_id][list_name] = markup_by_page[page_id].pop(rule.name)
            rule.name = list_name
            count += 1
            print rule.toJson()
            print "==============="

        print rules.toJson()
        return rules

if __name__ == '__main__':

    tree = Tree()

    import codecs
    with codecs.open('tmp.html', "r", "utf-8") as myfile:
        page_str = myfile.read().encode('utf-8')

    with codecs.open('tmp2.html', "r", "utf-8") as myfile:
        page_str2 = myfile.read().encode('utf-8')

    from landmark_ml.learning.TreeListLearner import TreeListLearner
    (paths, pg_mgr) = TreeListLearner.learn_lists({'tmp1.html': page_str, 'tmp2.html': page_str2}, filter_tags_by='div')

    markup_by_page = TreeListLearner.create_row_markups(paths, pg_mgr)

    import json
    print "=== MARKUP ==="
    print json.dumps(markup_by_page, sort_keys=True, indent=2, separators=(',', ': '))

    print "== learn sub rules =="
    rules = tree.learn_list_rules(markup_by_page, pg_mgr)

    print "== testing rules page 1 =="
    extraction_list = rules.extract(page_str)

    # from extraction.Landmark import flattenResult
    print json.dumps(extraction_list, sort_keys=True, indent=2, separators=(',', ': '))

    print "== testing rules page 2 =="
    extraction_list = rules.extract(page_str2)

    # from extraction.Landmark import flattenResult
    print json.dumps(extraction_list, sort_keys=True, indent=2, separators=(',', ': '))



import sys
import json
import string
import re
import random

from landmark_ml.util import PageManagementUtilities
from learning.PageManager import PageManager
from landmark_ml.learning import TreeListLearner


class SlotComparator(object):

    def __init__(self):
        self.__min_page_occurrences = 7

    def cluster_slots(self, page_directory, write_debug_files=False):
        pages = PageManagementUtilities().load_pages(page_directory, limit=250)

        print "THERE ARE %d PAGES" % len(pages)

        page_mgr = PageManager(write_debug_files=write_debug_files)

        for page in pages:
            page_content = pages[page]
            page_mgr.addPage(page, page_content)

        # first, do some processing to get the visible chunks and the FULL prefix tree
        visible_structure = page_mgr.getVisibleTokenStructure()
        tl = TreeListLearner()
        (ptree, paths_to_vis_text, path_to_invis_toks) = tl.prefix_tree(visible_structure)  # , tag_only=True)

        # make this into a by-page structure, since we will want to use that for counts, etc.
        visible_text_by_page = {}
        for entry in visible_structure:
            page_id = entry['page_id']
            vis_text = entry['visible_token_buffer']
            invis_tokens = entry['invisible_token_buffer_before']
            loc = entry['first_vis_token'].token_location

            if page_id not in visible_text_by_page:
                visible_text_by_page[page_id] = []
            visible_text_by_page[page_id].append((vis_text, loc, invis_tokens))

        # using the by-page structure, get the chunks and its left and right neighbors
        chunk_neighbors = {}
        for page in visible_text_by_page:
            # need to order chunks so for any chunk we can get it's left and right neighbors
            # note that a chunk may occur more than once per page, so we need it to be a list
            by_location = sorted([(a[1], a[0], a[2]) for a in visible_text_by_page[page]])  # now it's (loc, txt), (loc, txt)
            for idx in range(len(by_location)):
                (location, txt, invis_preceding) = by_location[idx]
                txt = txt.strip()
                if txt not in chunk_neighbors:
                    chunk_neighbors[txt] = []
                before_neighbor = '<START>'
                after_neighbor = '<END>'

                invis_after = ''

                if idx > 0:
                    before_neighbor = by_location[idx - 1][1]
                if idx < len(by_location) - 1:
                    after_neighbor = by_location[idx + 1][1]
                    invis_after = by_location[idx + 1][2]
                chunk_neighbors[txt].append((before_neighbor, after_neighbor, invis_preceding, invis_after))

        # now, prune the full prefix tree into the prefixes we care about (those that occur enough)
        tl.set_min_page_occurrences(self.__min_page_occurrences)
        valid_prefixes = tl.prefix_tree_to_paths(ptree)

        # do the structure to get all the visible chunks associated with the prefixes
        prefixes_with_vis = tl.get_visible_text_for_prefix(valid_prefixes, paths_to_vis_text)

        for prefix in prefixes_with_vis:
            print "== %s ==" % prefix
            cluster_for_prefix = prefixes_with_vis[prefix]
            cluster_size = float(len(cluster_for_prefix))

            if cluster_size < self.__min_page_occurrences:
                continue

            cluster_members = {}
            for c in cluster_for_prefix:
                count = cluster_for_prefix.count(c)
                prop = float(count) / cluster_size
                cluster_members[c] = {
                    'raw_count': count,
                    'proportion': prop
                }

            # maybe we want to take the top X by count?? Look at those??
            top_x = 10
            inverted_by_count = {}
            for p in cluster_members:
                cnt = cluster_members[p]['raw_count']
                if cnt not in inverted_by_count:
                    inverted_by_count[cnt] = []
                inverted_by_count[cnt].append(p)

            keepers = []
            for cnt in sorted(inverted_by_count.keys(), reverse=True):
                keepers.extend(inverted_by_count[cnt])
                if len(keepers) > top_x:
                    break
            printers = keepers[:top_x]

            prev_chunk_coverage = {}
            tot_coverage = 0
            for p in printers:

                #if cluster_members[p]['raw_count'] < self.__min_page_occurrences:
                #    continue

                chunk_ngs = chunk_neighbors[p]
                prev_chunks = [a[0] for a in chunk_ngs if a[2].find(prefix) > -1]  # only care about chunks preceding that have prefix between them

                for pc in prev_chunks:
                    if pc not in prev_chunk_coverage:
                        prev_chunk_coverage[pc] = 0
                    prev_chunk_coverage[pc] += 1
                    tot_coverage += 1
                print "\tVALUE: "+p+" -- OCCURS: (%d/%d)" % (cluster_members[p]['raw_count'], int(cluster_size))#, str(prev_chunks))
            if len(cluster_members.keys()) > 10:
                print "\tand (%d) more" % (len(cluster_members.keys()) - 10)

            print "-- prev chunks --"
            for pc in prev_chunk_coverage:
                print "\t\t%s (%d/%d)" % (pc.encode('ascii', 'ignore'), prev_chunk_coverage[pc], tot_coverage)
            print "\n\n"

        #why are we missing viz text?
        # its ok if our clusters are too precise...


    def cluster_slots_old(self, page_directory, write_debug_files=False):
        pages = PageManagementUtilities().load_pages(page_directory, limit=250)

        print "THERE ARE %d PAGES" % len(pages)

        page_mgr = PageManager(write_debug_files=write_debug_files)

        for page in pages:
            page_content = pages[page]
            page_mgr.addPage(page, page_content)

        visible_structure = page_mgr.getVisibleTokenStructure()
        tl = TreeListLearner()
        tl.set_min_page_occurrences(self.__min_page_occurrences)
        (ptree, paths_to_vis_text, path_to_invis_toks) = tl.prefix_tree(visible_structure) #, tag_only=True)

        valid_prefixes = tl.prefix_tree_to_paths(ptree)


        # invert the paths to visible text so it's keyed on chunks...
        prefix_paths_for_chunks = {}
        for path in paths_to_vis_text:
            chnks = paths_to_vis_text[path]
            for chnk in chnks:
                if chnk not in prefix_paths_for_chunks:
                    prefix_paths_for_chunks[chnk] = []
                if path not in prefix_paths_for_chunks[chnk]:
                    prefix_paths_for_chunks[chnk].append(path)

        # make this into a by-page structure, since we will want to use that for counts, etc.
        visible_text_by_page = {}
        for entry in visible_structure:
            page_id = entry['page_id']
            vis_text = entry['visible_token_buffer']
            loc = entry['first_vis_token'].token_location

            if page_id not in visible_text_by_page:
                visible_text_by_page[page_id] = []
            visible_text_by_page[page_id].append((vis_text, loc))

        # ok, for each piece of visible text, let's get some features going
        visible_text_features = {}
        for page in visible_text_by_page:
            visible_text_features[page] = {}

            uniq_chunks = set([tup[0] for tup in visible_text_by_page[page]])

            # need to order chunks so for any chunk we can get it's left and right neighbors
            # note that a chunk may occur more than once per page, so we need it to be a list
            by_location = sorted([(a[1], a[0]) for a in visible_text_by_page[page]]) # now it's (loc, txt), (loc, txt)
            chunk_neighbors = {}
            for idx in range(len(by_location)):
                (location, txt) = by_location[idx]
                if txt not in chunk_neighbors:
                    chunk_neighbors[txt] = []
                before_neighbor = '<START>'
                after_neighbor = '<END>'
                if idx > 0:
                    before_neighbor = by_location[idx-1][-1]
                if idx < len(by_location) - 1:
                    after_neighbor = by_location[idx+1][-1]
                chunk_neighbors[txt].append((before_neighbor, after_neighbor))


            for chunk in uniq_chunks:
                visible_text_features[page][chunk] = {
                    'chunk_length': len(chunk),
                    'num_tokens': len(chunk.strip().split()),
                    'occurences_on_page': self.occurences_on_page(chunk, visible_text_by_page[page]),
                    'pct_pages_appears': self.pct_pages_appears(chunk, visible_text_by_page),
                    'number_periods': self.count_character_occurrence(chunk, "."),
                    'number_dashes': self.count_character_occurrence(chunk, "-"),
                    'number_commas': self.count_character_occurrence(chunk, ","),
                    'number_fwd_slashes': self.count_character_occurrence(chunk, "/"),
                    'number_back_slashes': self.count_character_occurrence(chunk, "\\"),
                    'number_dollar_signs': self.count_character_occurrence(chunk, "$"),
                    'tree_paths': prefix_paths_for_chunks[chunk],
                    'pct_non_letter_chars': self.pct_non_chars(chunk),
                    'chunk_before': [a[0] for a in chunk_neighbors[chunk]],
                    'chunk_after': [a[-1] for a in chunk_neighbors[chunk]],
                    'page': page, # long story, but makes it easier to invert by chunk below...
                }

        # ok, so now let's invert this? Want to look at each chunk...
        chunk_features = {}
        for page in visible_text_features:
            for chunk in visible_text_features[page]:
                if chunk not in chunk_features:
                    chunk_features[chunk] = {}
                for feature in visible_text_features[page][chunk]:
                    if feature not in chunk_features[chunk]:
                        chunk_features[chunk][feature] = []
                    chunk_features[chunk][feature].append(visible_text_features[page][chunk][feature])


        # so, dedupe the values (helps take care of some that will be the same, like length, across all of them
        # really lazy bc I could do something smarter about all of these above
        chunk_features_final = {}
        for chunk in chunk_features:
            chunk_features_final[chunk] = {}
            for feature in chunk_features[chunk]:
                # going to uniqefy each value
                try:
                    # in a try bc list of listst causes problems bc its unhashable
                    chunk_features_final[chunk][feature] = list(set(chunk_features[chunk][feature]))
                except:
                    # so if it's a list of lists, flatten it, and then uniquefy
                    chunk_features_final[chunk][feature] = list(set([item for sublist in chunk_features[chunk][feature] for item in sublist]))

        print json.dumps(chunk_features_final)
        print "-----------------------------"
        print "-----------------------------"
        for chunk in chunk_features_final:
            if chunk_features_final[chunk]['pct_pages_appears'][0] > 0.05: # 0.7: # TODO: Perhaps make this a fixed number, rather than PCT (5, 7)
                do_print = False

                output = chunk+"\n"
                output += '\t'+' -- '.join(chunk_features_final[chunk]['chunk_after'][:5])+" +("+str(len(chunk_features_final[chunk]['chunk_after']) - 5)+") more\n"

                # if the text after has many values, then maybe chunk is some sort of attribute text?
                # or look at how many times each of the after texts appaers across teh site? If low, then it changes
                # a lot... so you could do an average? see if it's low and then that makes current chunk look like
                # an attribute?
                # Otherwise it could be "static page info" or site info? (like categories, such as states, or headers)
                change_estimator =\
                    float(len(chunk_features_final[chunk]['chunk_after'])) / float(len(visible_text_by_page.keys()))

                if change_estimator >= 0.01: #0.1:
                    do_print = True
#                    print "\t -- ATTRIBUTE"

                    #look at the invisible text between this visible token and the ones that follow it
                    #so you need to somehow figure that out, and count thsoe paths

                    path_counts = {}
                    for chnk in chunk_features_final[chunk]['chunk_after']:
                        paths = chunk_features_final[chnk]['tree_paths']
                        for pth in paths:
                            if pth not in path_counts:
                                path_counts[pth] = 0.0
                            path_counts[pth] += 1.0
                    for p in path_counts:
                        path_counts[p] = path_counts[p] / float(len(chunk_features_final[chunk]['chunk_after']))

                    output += "\tPATH COUNTS:\n"
                    for p in path_counts:
                        output += "\t\t"+p+" -- "+str(path_counts[p])+"\n"

                if do_print:
                    print output
                    print '......'

#        so if you are flagged as data, essentially, if the guy preceding you is a data flag, then your value is data...

    def pct_pages_appears(self, chunk, visible_text_by_page):
        seen_pages = 0
        for page in visible_text_by_page:
            cnt = self.occurences_on_page(chunk, visible_text_by_page[page])
            if cnt > 0:
                seen_pages += 1
        return float(seen_pages) / float(len(visible_text_by_page.keys()))

    def occurences_on_page(self, chunk, page_chunks):
        count = 0
        for (p_chunk, location) in page_chunks:
            if p_chunk == chunk:
                count += 1
        return count

    def count_character_occurrence(self, chunk, character):
        return chunk.count(character)

    def strip_punct(self, chunk):
        return str(chunk.encode('ascii', 'ignore')).translate(string.maketrans("", ""), string.punctuation)

    def pct_non_chars(self, chunk):
        return 1.0 - float(len(self.strip_punct(chunk))) / float(len(chunk))


if __name__ == '__main__':
    s = SlotComparator()
    s.cluster_slots(sys.argv[1])



import os
import codecs
import random

class PageManagementUtilities(object):
    def __init__(self):
        pass

    def load_pages(self, page_file_dir, limit=1000000):
        pages = {}
        print page_file_dir
        for subdir, dirs, files in os.walk(page_file_dir):
            for the_file in files:
                if the_file.startswith('.'):
                    continue
                with codecs.open(os.path.join(subdir, the_file), "r", "utf-8") as myfile:
                    page_str = myfile.read().encode('utf-8')
                pages[the_file] = page_str

        if len(pages.keys()) > limit:
            # if we have too many pages, then we pull out a random sample of pages, of size limit
            page_keys = pages.keys()
            random.shuffle(page_keys)  # randomly order the pages
            #  pull out a subset of pages (0 to limit) from this shuffled set
            final_pages = dict([(fname, pages[fname]) for fname in page_keys[:limit]])
            return final_pages

        return pages




import urllib3
import sys
from elasticsearch import Elasticsearch
import getopt
import codecs
import re
import json
urllib3.disable_warnings()

user_password = "user:password"
domain = "domain"
gt_regex = 'gt_regex'
limit = None

usage = "get_all_pages_for_sites_for_groundtruth -u " + user_password + " -d " + domain + " -g " + gt_regex

try:
    opts, args = getopt.getopt(sys.argv[1:], "hu:d:g:l:")
except getopt.GetoptError:
    print usage
    sys.exit(2)

for opt, arg in opts:
    if opt == '-h':
        print usage
        sys.exit()
    elif opt in "-u":
        user_password = arg
    elif opt in "-d":
        domain = arg
    elif opt in "-g":
        gt_regex = arg
    elif opt in "-l":
        limit = int(arg)

if user_password == "user:password" or domain == "domain" or gt_regex == 'gt_regex':
    print usage
    sys.exit(2)

ground_truth_regex = re.compile(gt_regex)

all_output_file = '_'+domain+'_all_urls.txt'
groundtruth_output_file = '_'+domain+'_gt.jl'
other_output_file = '_'+domain+'_not_gt.jl'

cdr = "https://" + user_password + "@els.istresearch.com:19200"
es = Elasticsearch(cdr, timeout=30)

# Query to get 1000 urls at a time
urls_query = {
   "query": {
      "match": {
          "url.domain": domain
      }
   },
   
   "_source": {
        "include": [
            "url",
            "raw_content"
        ]
    }
}

# Initialize the scroll
page = es.search(
  scroll = '2m',
  search_type = 'scan',
  size = 10,
  body = urls_query,
  request_timeout=30)
sid = page['_scroll_id']
scroll_size = page['hits']['total']

gt_links = {}
other_links = {}

# Start scrolling
done = False
with codecs.open(all_output_file, "w", encoding='utf-8') as all_site_file:
    while (scroll_size > 0 and not done):
        print "Scrolling..."
        page = es.scroll(scroll_id = sid, scroll = '2m')
        # Update the scroll ID
        sid = page['_scroll_id']
        # Get the number of results that we returned in the last scroll
        scroll_size = len(page['hits']['hits'])
        print "scroll size: " + str(scroll_size)
        # Do something with the obtained page
        for hit in page['hits']['hits']:
            json_vals = {}
            _id = hit['_id']
            url = hit['_source']['url']
            if 'raw_content' in hit['_source']:
                raw_content = hit['_source']['raw_content']
            all_site_file.write(url + "\n")
            
            if ground_truth_regex.search(url):
                gt_links[url] = hit
                if limit:
                    if len(gt_links.keys()) >= limit:
                        done = True
                        break
            else:
                other_links[url] = hit
                
with codecs.open(groundtruth_output_file, "w", encoding='utf-8') as gt_file:
    for url in gt_links:
        gt_file.write(json.dumps(gt_links[url]) + "\n")
    gt_file.close()
    
with codecs.open(other_output_file, "w", encoding='utf-8') as other_file:
    for url in other_links:
        other_file.write(json.dumps(other_links[url]) + "\n")
    other_file.close()
                        


import sys
import getopt
import json
import codecs
import os
import urllib
import cgi
from landmark_extractor.extraction.Landmark import RuleSet

INDEX_TEMPLATE = '\
<!DOCTYPE html>\
<html>\
<head>\
    <title>[[TITLE]]</title>\
    <script src="jquery.1.11.2.js" type="text/javascript"></script>\
    <script src="bootstrap.min.3.3.1.js" type="text/javascript"></script>\
    <script src="jquery.watable.js" type="text/javascript" charset="utf-8"></script>\
    <link rel="stylesheet" href="bootstrap.min.3.3.1.css" />\
    <link rel="stylesheet" href="watable.css"/>\
    <style type="text/css">\
        body { padding: 30px; font-size: 12px }\
        .watable * {white-space: inherit !important;}\
    </style>\
</head>\
<body>\
    <h2>[[TITLE]]</h2>\
    <div id="example1" style="width:auto"></div>\
    <script type="text/javascript">\
        var cols =\
            [[COLUNMS]];\
        var rows =\
            [[ROWS]];\
        var data = {\
            cols: cols,\
            rows: rows\
        };\
        $(document).ready( function() {\
                $("#example1").WATable({\
                    data: data,\
                    pageSizes: [10,25,50,100,250],\
                    columnPicker: true\
                });\
        });\
    </script>\
</body>\
</html>'

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg
        
def run_extraction_check(page_file_dir, output_dir, rules, show_empties = True, is_cdr_format = False):
    output_str = os.path.join(os.path.abspath(output_dir), 'eval.html')
    
    titles = rules.names()
    titles.insert(0, 'page')
    
    cols = {}
    rows = []
        
    total_count = 0
    counts = {}
    for name in rules.names():
        counts[name] = 0
    
    files = [f for f in os.listdir(page_file_dir) if os.path.isfile(os.path.join(page_file_dir, f))]
    for the_file in files:
        if the_file.startswith('.'):
            continue
        
        with codecs.open(os.path.join(page_file_dir, the_file), "r", "utf-8") as myfile:
            if is_cdr_format:
                cdr_str = myfile.read().encode('utf-8')
                page_str = json.loads(cdr_str)['raw_content']
            else:
                page_str = myfile.read().encode('utf-8')
        total_count = total_count + 1
        
        extraction = rules.extract(page_str)
#                 if not rules.validate(extraction):
#                     continue
        
        results = []
        results.append(urllib.unquote(the_file))
        
        row = {}
        row['page'] = os.path.abspath(page_file_dir)+"/"+urllib.pathname2url(the_file)
#                 row['page'] = subdir+"/"+urllib.pathname2url(the_file)
        
        for name in rules.names():
            found = False
            if name in extraction:
                if extraction[name]:
                    extract = extraction[name]['extract']
                    if 'sequence' in extraction[name]:
                        extract = 'LIST<br>----<br>'
                        for sequence_extract in extraction[name]['sequence']:
                            extract = extract + cgi.escape(str(sequence_extract['sequence_number']) + ') ' + sequence_extract['extract']) + '<br>'
                    else:
                        extract = cgi.escape(extract)
                    
                    if extract:
                        results.append(extract)
                        row[name] = extract
                        counts[name] = counts[name] + 1
                        found = True
                    
            if not found:
                row[name] = ''
                key = name+'Cls'
                row[key] = 'red'
                results.append('NULL')
                
#                 wr.writerow(results)
        rows.append(row)
    
    #make the cols for the UI
    index = 1
    for title in titles:
        col = {}
        col['index'] = index
        col['type'] = 'string'
        
        if index == 1:
            col['sortOrder'] = 'asc'
            col['format'] = '<span style="word-break:break-all;"><a href="file://{0}" target="_blank">{0}</a></span>'
        else:
            col['friendly'] = title + ' [' + str(counts[title]) + '/' + str(total_count) +']'
            col['tooltip'] = rules.get(title).toolTip()
        cols[title] = col
        
        index = index + 1
    
    index_output = INDEX_TEMPLATE.replace('[[TITLE]]', page_file_dir).replace('[[COLUNMS]]', json.dumps(cols)).replace('[[ROWS]]', json.dumps(rows))
    
    with codecs.open(output_str, "w", "utf-8") as myfile:
        myfile.write(index_output)

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "h", ["help"])
        except getopt.error, msg:
            raise Usage(msg)
        
        #read the directory location from arg0
        page_file_dir = args[0]
        
        output_dir = args[1]
        
        #read the rules from arg1
        json_file_str = args[2]
        with codecs.open(json_file_str, "r", "utf-8") as myfile:
            json_str = myfile.read().encode('utf-8')
        
        json_object = json.loads(json_str)
        rules = RuleSet(json_object)
        
        run_extraction_check(page_file_dir, output_dir, rules)
        
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2

if __name__ == '__main__':
    sys.exit(main())


import sys
import getopt
from landmark_ml.learning.PageManager import PageManager
import codecs
import os

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "h", ["help"])
        except getopt.error, msg:
            raise Usage(msg)
        
        pageManager = PageManager()
        page_file_dir = args[0]
        
        for subdir, dirs, files in os.walk(page_file_dir):
            for file_pointer in files:
                if file_pointer.startswith('.'):
                    continue
                
                with codecs.open(os.path.join(subdir, file_pointer), "r", "utf-8") as myfile:
                    page_str = myfile.read().encode('utf-8')
                    
                pageManager.addPage(file_pointer, page_str)
        
        pageManager.learnStripes()
        
        count = 0
        for stripe in pageManager.getStripes():
            print str(count) + ": " + str(stripe)
            count = count + 1
        
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2

if __name__ == '__main__':
    sys.exit(main())


# No tests yet, so this is just a placeholder to show the API
import sys
import codecs
import os
from landmark_ml.learning.TreeListLearner import TreeListLearner

class DivListLearner(object):

    def run(self, train_pages, test_pages = None):
        t = TreeListLearner()
        rules, markup = t.learn_list_extractors(train_pages)

        return rules, markup

    def run_one(self, content):
        t = TreeListLearner()
        t.lists_on_single_page(content)

def main(argv):
    page_file_dir = argv[0]
    train_pages = {}
    for subdir, dirs, files in os.walk(page_file_dir):
        for the_file in files:
            if the_file.startswith('.'):
                continue
            
            with codecs.open(os.path.join(subdir, the_file), "r", "utf-8") as myfile:
                page_str = myfile.read().encode('utf-8')
            train_pages[the_file] = page_str
    
    d = DivListLearner()
    
    d.run(train_pages, train_pages)

if __name__ == "__main__":
    main(sys.argv[1:])



class TreeNode(object):
    def __init__(self, value, parent, path_to_node):
        self.__value = value
        self.__parent = parent
        self.__path_to_node = path_to_node
        self.__children = {}  # key = text for child, value = count
        self.__metaData = []  # flexible list of meta-data associated with this node

    def getPath(self):
        return self.__path_to_node

    def getValue(self):
        return self.__value

    def getChildren(self):
        return self.__children

    def getMetaData(self):
        return self.__metaData

    def addChild(self, child):
        if child not in self.__children:
            self.__children[child] = 0.
        self.__children[child] += 1.

    def deleteChild(self, child):
        del self.__children[child]

    def updateMetaData(self, meta):
        self.__metaData.append(meta)

    def getMetaDataForKey(self, key):
        values = []
        for struct in self.__metaData:
            values.append(struct[key])
        return values

    def addChildWithMetaData(self, child, key, value):
        self.addChild(child)
        self.updateMetaData(key, value)

    def __str__(self):
        output = "NODE: %s | CHD: %s | MT: %s | PTH: %s" % (self.__value, str(self.__children),
                                                            str(self.getMetaDataForKey('visible_text')), self.__path_to_node)
        return output

    # def __str__(self):
    #     output = ''
    #     output += "NODE: %s\n" % self.__value
    #     output += "CHILD: %s\n" % str(self.__children)
    #     output += "META: %s\n" % str(self.__metaData)
    #     output += "..........\n"
    #     return output


import json
import os
import codecs
from landmark_extractor.extraction.Landmark import flattenResult, RuleSet

rules_file_string = 'step01_rules.json'
output_file_string = 'step01_extractions.jl'
pages = [
         {'_cdr': 'page_1.html', '_url': 'url1'},
         {'_cdr': 'page_3.html', '_url': 'url2'},
         {'_cdr': 'page_4.html', '_url': 'url3'},
         {'_cdr': 'page_5.html', '_url': 'url4'}
         ]

json_line = {}
directory = '/cluster_dir/'

with codecs.open(os.path.join(directory, rules_file_string), "r", "utf-8") as myfile:
    json_str = myfile.read().encode('utf-8')
json_object = json.loads(json_str)
rules = RuleSet(json_object)

jl_file = os.path.join(directory, output_file_string)
with codecs.open(jl_file, "w", encoding='utf-8') as extractions_jl:
    for page in pages:
        json_line = {}
        json_line["_cdr"] = page["_cdr"]
        json_line["_url"] = page['_url']
        
        with codecs.open(os.path.join(directory, json_line["_cdr"]), "r", "utf-8") as myfile:
            page_str = myfile.read().encode('utf-8')
            
            extraction_list = rules.extract(page_str)
            json_line.update(flattenResult(extraction_list))
            extractions_jl.write(json.dumps(json_line))
            extractions_jl.write('\n')



import sys
import getopt
from landmark_ml.learning.PageManager import PageManager
import codecs
import os
import json

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "h", ["help"])
        except getopt.error, msg:
            raise Usage(msg)
        
        #read the directory location from arg0
        page_file_dir = args[0]
        
        pageManager = PageManager()
        
        for subdir, dirs, files in os.walk(page_file_dir):
            for the_file in files:
                if the_file.startswith('.'):
                    continue
                
                with codecs.open(os.path.join(subdir, the_file), "r", "utf-8") as myfile:
                    page_str = myfile.read().encode('utf-8')
                    
                pageManager.addPage(the_file, page_str)
                
        pageManager.learnStripes()
        
        #Read the markups from a file...
        markups_file = args[1]
        with codecs.open(os.path.join('', markups_file), "r", "utf-8") as myfile:
            markup_str = myfile.read().encode('utf-8')
        markups = json.loads(markup_str)

        print markups

        shortest_pairs = []
        all_bounding_stripes = []
        for markup in markups:
            print "mark:", markup
            page_id = markup
            title = markups[markup]['title']['extract']
            print "title:", title
            shortest_pair = pageManager.getPossibleLocations(page_id, title, False)
            print "shortest_pair:", shortest_pair

            bounding_stripes = pageManager.getAllBoundingStripes(page_id, shortest_pair)

            all_bounding_stripes.append(bounding_stripes)

        print "all bounding stripes:", all_bounding_stripes

        valid_bounding_stripes = pageManager.getValidBoundingStripes(all_bounding_stripes)

        print "valid stripes:", valid_bounding_stripes
        
        begin_stripes = pageManager.getNextLevelStripes(valid_bounding_stripes[0],'begin')

        print "begin stripes:", begin_stripes

        end_stripes = pageManager.getNextLevelStripes(valid_bounding_stripes[0], 'end')

        print "end stripes:", end_stripes

    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2

if __name__ == '__main__':
    sys.exit(main())


import networkx as nx
import re
import json
from landmark_ml.learning.PageManager import PageManager
from landmark_ml.learning.Tree import Tree

class TreeListLearner(object):

    def __init__(self):
        self.__minEdgeWeight = 2
        self.__DEBUG = False

    @staticmethod
    def learn_lists(contents, filter_tags_by=None):
        tree = Tree()
        pg = PageManager()
        for pageid in contents:
            content = contents[pageid]
            pg.addPage(pageid, content)
        structure_data = pg.getVisibleTokenStructure(data_as_strings=False, data_as_tokens=True)

        # want to keep track of the first token location for visible text and the last token location for visible text
        # we can do this pretty easily as meta-data for our node
        for struct in structure_data:
            visible_text = ''.join([a.token for a in struct['visible_token_buffer']])
            first_vis_token_loc = struct['visible_token_buffer'][0].token_location
            last_vis_token_loc = struct['visible_token_buffer'][-1].token_location

            if filter_tags_by is None:
                tokens_for_path = struct['invisible_token_buffer_before']
            else:
                if filter_tags_by == 'space_punct':
                    token_html = ''.join(struct['invisible_token_buffer_before']).replace("><", "> <")
                    # simple way... replace all the punct we care about (=, ., /, :) with space
                    punct_to_split = ["/", "=", "-", ":"]
                    for p in punct_to_split:
                        token_html = token_html.replace(p, " ")
                    print token_html
                    tokens_for_path = token_html.split()
                else:
                    potential_toks = re.findall("(<.+?>)", ''.join(struct['invisible_token_buffer_before']))
                    tokens_for_path = [a for a in potential_toks if a.startswith("<" + filter_tags_by)]

            meta_struct = {
                'visible_text': visible_text,
                'first_vis_token_loc': first_vis_token_loc,
                'last_vis_token_loc': last_vis_token_loc,
                'page_id': struct['page_id']
            }

            tokens_for_path.reverse()
            # how do we want to process these tokens right now? Can consider all the tokens
            tree.add_node_set(tokens_for_path, meta_struct)

        tree.prune()
        paths = tree.valid_paths()
        for path in paths:
            print path.replace(tree.getPathDelimeter(), '')
            for m in paths[path]:
                if m['page_id'] == 'tmp1.html':
                    print "\t" + m['visible_text']+" "+m['page_id']+" "+str(m['first_vis_token_loc'])

        return paths, pg

    """
        Given the rows we extract, separate them into clusters where you have overlapping rows or not.
        This is the first step to finding interleaving...

        Once we find the interleaving, we merge them in (via common parts of the paths), and create
        the lists.

        From that, we make markup and that's what we give back
    """

    @staticmethod
    def create_row_markups(valid_rows, page_manager):
        valid_rows_by_page = {}
        for path in valid_rows:
            for row in valid_rows[path]:
                pg = row['page_id']
                if pg not in valid_rows_by_page:
                    valid_rows_by_page[pg] = {}
                if path not in valid_rows_by_page[pg]:
                    valid_rows_by_page[pg][path] = []
                valid_rows_by_page[pg][path].append(row)

        markup_by_page = {}
        for page in valid_rows_by_page:
            all_tokens_list = page_manager.getPage(page).tokens

            markup_by_page[page] = {}
            valid_rows_for_page = valid_rows_by_page[page]
            # print "VALID ROWS FOR: %s" % page
            # print str(valid_rows_for_page)

            earliest_latest_row_locations = {}
            for path in valid_rows_for_page:  # the path defines the row...
                earliest = -1
                latest = -1

                for row in valid_rows_for_page[path]:
                    s_loc = row['first_vis_token_loc']
                    e_loc = row['last_vis_token_loc']
                    if earliest == -1:
                        earliest = row['first_vis_token_loc']
                        latest = row['last_vis_token_loc']
                        continue

                    if s_loc < earliest:
                        earliest = s_loc
                    if e_loc > latest:
                        latest = e_loc

                earliest_latest_row_locations[path] = (earliest, latest)

            # print str(earliest_latest_row_locations)

            overlaps = []
            for pth in earliest_latest_row_locations:
                begin = earliest_latest_row_locations[pth][0]
                end = earliest_latest_row_locations[pth][1]
                if begin == -1 or end == -1:  # ill defined locations
                    continue
                if len(overlaps) == 0:  # first guy...
                    overlaps.append([pth])
                    continue

                overlap_clust = -1
                for clust_id in range(len(overlaps)):
                    cluster = overlaps[clust_id]
                    for cpath in cluster:  # could probably just find min and max of cluster and check w/ that, but easier for now...
                        p_begin = earliest_latest_row_locations[cpath][0]
                        p_end = earliest_latest_row_locations[cpath][1]

                        #  now, see if there is not  overlap...
                        if p_end < begin or p_begin > end:
                            continue
                        overlap_clust = clust_id

                if overlap_clust == -1:
                    overlaps.append([pth])
                else:
                    overlaps[overlap_clust].append(pth)

            # print "OVERLAPS"
            # print str(overlaps)

            for clust in overlaps:
                # print "===oo00 CLUSTER 00oo==="
                # print clust
                path_for_start = ""
                # left most, largest row is the beginning, so use that one as A's'
                rows_start_location = 999999999999
                rows_end_location = 0

                # first, find the member with the most rows
                max_rows = max([len(valid_rows_for_page[member]) for member in clust])

                # Ok, so the HTML between rows could have been messed up before bc we didn't know that these were
                # overlapping lists. For instance, the first row could be alone and now it's merged, so let's remake
                # the html between...

                by_location_tuples = []  # it will be (start, end, path) just to make it super easy to build the markup
                # then once we have this filled in, and we know which path demarcates each row, we simply sort
                # then iterate thru making up the rows...
                for member in clust:
                    num_rows = len(
                        valid_rows_for_page[member])  # its ok that its combined across pages... bc aggregate number
                    # print "\t--> (%d, %d): %d" % (earliest_latest_row_locations[member][0],
                    #                               earliest_latest_row_locations[member][1], num_rows)
                    # print "\t\t PATH: " + member

                    for b in valid_rows_for_page[member]:
                        by_location_tuples.append(
                            (b['first_vis_token_loc'], b['last_vis_token_loc'], member, b['page_id']))
                        # print "\t\t\t%s %s %s %s" % (str(b['first_vis_token_loc']), b['visible_text'],
                        #                              str(b['last_vis_token_loc']), b['page_id'])

                    if num_rows == max_rows:
                        if earliest_latest_row_locations[member][0] < rows_start_location:
                            rows_start_location = earliest_latest_row_locations[member][0]
                            path_for_start = member
                        if earliest_latest_row_locations[member][1] > rows_end_location:
                            # TODO: BA I think we need to extend this "if it still has overlap with the others??"
                            rows_end_location = earliest_latest_row_locations[member][1]

                print ">> Row starts at: %d and ends at %d (%s) " % (rows_start_location, rows_end_location, path_for_start)
                sorted_loc_triples = sorted(by_location_tuples)
                # print "SORTED LOCATION TRIPLES"
                # print str(by_location_tuples)

                # now we know which path is the "start" and where each one begins and ends, so let's make the structure
                # first we want to find all entries of path_for_start
                rows = [(tpl[0], tpl[3]) for tpl in sorted_loc_triples if tpl[2] == path_for_start]
                for idx in range(len(rows)):
                    lc = rows[idx][0]
                    if idx < len(rows) - 1:
                        lc_next = rows[idx + 1][0]
                        end = lc_next - 1  # we go till right before this guy
                    else:
                        end = rows_end_location

                    # TODO: BA This is where we "strip" the invisible stuff from the end
                    row_text_offset_end = 0
                    for token in reversed(all_tokens_list[lc:end+1]):
                        if token.visible:
                            break
                        row_text_offset_end += 1
                        # if token.token == '<':
                        #     break
                    end = end - row_text_offset_end

                    # TODO: BA And we add some stuff to the end of the last one because it causes some issues...
                    # if idx == len(rows)-1:
                    #     row_text_offset_last = 0
                    #     for token in all_tokens_list[end + 1:]:
                    #         if token.visible:
                    #             break
                    #         row_text_offset_last += 1
                    #         # if token.token == '<':
                    #         #     break
                    #     end = end + row_text_offset_last


                    # get the location info between...
                    # from all tokens, get all the tokens between and get the string
                    # then add the markup (include the sequency number)
                    markup_value = all_tokens_list.getTokensAsString(lc, end + 1, whitespace=True)
                    markup_data = {
                        'extract': markup_value,
                        'sequence_number': idx + 1,
                        'starting_token_location': lc,
                        'ending_token_location': end
                    }
                    # print "%s: (%s)" % (page, str(markup_data))

                    if path_for_start not in markup_by_page[page]:
                        markup_by_page[page][path_for_start] = {
                            'sequence': []
                        }

                    markup_by_page[page][path_for_start]['sequence'].append(markup_data)

        for page in markup_by_page:

            for path_for_start in markup_by_page[page]:
                min_location = 9999999999
                max_location = -1
                for idx in range(len(markup_by_page[page][path_for_start]['sequence'])):
                    if markup_by_page[page][path_for_start]['sequence'][idx]['starting_token_location'] < min_location:
                        min_location = markup_by_page[page][path_for_start]['sequence'][idx]['starting_token_location']
                    if markup_by_page[page][path_for_start]['sequence'][idx]['ending_token_location'] > max_location:
                        max_location = markup_by_page[page][path_for_start]['sequence'][idx]['ending_token_location']
                markup_by_page[page][path_for_start]['starting_token_location'] = min_location
                markup_by_page[page][path_for_start]['ending_token_location'] = max_location

        return markup_by_page

#############################################
    # AFTER HERE IS OLD... SEE WHAT WE CAN DELETE
    #
##############################################


    # number of times you see something. If less than this, then prefix is pruned
    def set_min_page_occurrences(self, occs):
        self.__minEdgeWeight = occs

    """
        pageRepresentation is the invisible/visible data structure
        only_consider_tag lets you filter to just one tag type, like DIV
    """
    def prefix_tree(self, pageRepresentation, only_consider_tag=None, tag_only=False):
        ptree = {}
        path_to_visible_texts = {}
        path_to_first_invis_tokens = {}

        for tupl in pageRepresentation:
            invisible_token_string = tupl['invisible_token_buffer_before'].replace("> <", "><")
            invisible_tokens = re.findall("(<.+?>)", invisible_token_string)

            if only_consider_tag is not None:
                invisible_tokens = [a for a in invisible_tokens if a.startswith("<" + only_consider_tag)]
            else:
                # we don't care about end tags...
                invisible_tokens = [a for a in invisible_tokens if not a.startswith("</")]

            if tag_only:
                invisible_tokens = [re.search("\S+", tk).group(0) for tk in invisible_tokens]
                # and now get rid of end tags...
                invisible_tokens = [tk+">" for tk in invisible_tokens if tk.startswith("<")]

            path_string = ''.join(invisible_tokens)
            if path_string not in path_to_visible_texts:
                path_to_visible_texts[path_string] = []
            path_to_visible_texts[path_string].append(tupl['visible_token_buffer'])

            if path_string not in path_to_first_invis_tokens:
                path_to_first_invis_tokens[path_string] = []
            path_to_first_invis_tokens[path_string].append(tupl['first_invis_token'])

            invisible_tokens.append('VISIBLE')  # BC we are going to reverse and make this root

            # first, we want to process right to left...
            invisible_tokens.reverse()

            for depth in range(len(invisible_tokens)):
                if depth not in ptree:
                    ptree[depth] = {}
                if depth == 0:
                    if 'VISIBLE' not in ptree[depth]:
                        ptree[depth]['VISIBLE'] = {'count': 9999999, 'parent': ''}
                else:
                    node = invisible_tokens[depth]
                    if node not in ptree[depth]:
                        ptree[depth][node] = {}
                        ptree[depth][node] = {'count': 1, 'parent': invisible_tokens[depth - 1]}
                    else:
                        ptree[depth][node]['count'] += 1

        return ptree, path_to_visible_texts, path_to_first_invis_tokens

    def prefix_tree_to_paths(self, prefix_tree):
        #  basically go through the prefix tree, turn each path into a rule and see the visible text that follows it
        #  turn paths in the tree into results by looking at the visible text that follows each path

        # go from leaf to root
        G = nx.DiGraph()
        for i in prefix_tree.keys():
            if i == 0:
                continue
            else:
                if i == 1:
                    for node in prefix_tree[i]:
                        G.add_edge('VISIBLE', str(i) + "||" + node, weight=prefix_tree[i][node]['count'], label=prefix_tree[i][node]['count'])
                else:
                    for node in prefix_tree[i]:
                        G.add_edge(str(i - 1) + "||" + prefix_tree[i][node]['parent'], str(i) + "||" + node,
                                   weight=prefix_tree[i][node]['count'], label=prefix_tree[i][node]['count'])

        leaves = [x for x in G.nodes_iter() if G.out_degree(x) == 0]  # nodes with no out degree are leaves

        # note we have some disconnected trees, so there might not be a path... but...
        paths = []
        for leaf in leaves:
            has_path = nx.has_path(G, 'VISIBLE', leaf)
            if has_path:
                short_path = nx.shortest_path(G, 'VISIBLE', leaf)

                # leading divs
                leading_tags = [a for a in short_path if a != 'VISIBLE']
                leading_tags.reverse()
                paths.append(leading_tags)

        # first, create the path sets... note, any path set would share hte same first token
        path_sets = {} # key: first token of path, value: list of members
        for pth in paths:
            pth.reverse()
            first_tok = pth[0]
            if first_tok not in path_sets:
                path_sets[first_tok] = []
            path_sets[first_tok].append(pth)

        # now, see if the path set is "valid." A valid pathset is a pathset where at least one member is
        # valid (e.g., path from root to leaf has all edges occur at least once
        paths_to_keep = []
        for path_set_identifier in path_sets.keys():
            good_path_parts = []  # for holding paths where edges occur at least number of times we want
            for p in path_sets[path_set_identifier]:
                edge_data = [G.get_edge_data(p[i], p[i+1]) for i in range(len(p)) if i < len(p) - 1]
                tok_with_edge_data = zip(p, edge_data)
                keepers = [tupl[0] for tupl in tok_with_edge_data if tupl[1]['weight'] >= self.__minEdgeWeight]

                # TODO: If you are missing the first (last?) token, then it means you are breaking from VISIBLE...
                # why you end up with lists that are just one node and don't actually extract anything

                good_path_parts.append(keepers)

            # now, find the intersection of the guys in good path parts, this will be our final path
            final_keeper = []
            for i in range(len(good_path_parts)):
                if i == 0:
                    final_keeper = good_path_parts[i]
                else:
                    final_keeper = [z for z in good_path_parts[i] if z in final_keeper]
            final_keeper.reverse()  # reverse it back to what it looked like before
            if len(final_keeper) > 0:
                paths_to_keep.append(final_keeper)

        # finally, clean the tags
        cleaned_tags = []
        for pth in paths_to_keep:
            cleaned_tags.append([a.split("||")[-1] for a in pth])

        #nx.drawing.nx_pydot.write_dot(G, 'test.dot')
        return cleaned_tags

    """
        Sometimes for a given prefix in the tree we want the visible text associated with that prefix.
        This is not as clean as keep track when you actually generate the prefixes, but for now this should be
        sufficient.

        TODO: Do this the "hard" way to keep track of the viz text as you generate each prefix
    """
    def get_visible_text_for_prefix(self, valid_prefixes, paths_to_vis_text):
        prefixes_with_text = {}
        for prefix in valid_prefixes:  # these will define the clusters for now
            prefix_str = ''.join(prefix)
            cluster_for_prefix = []
            for path in paths_to_vis_text:
                path_no_ends = re.sub("</\w+>", "", path)
                if path_no_ends.find(prefix_str) > -1:
                    cluster_for_prefix.extend(paths_to_vis_text[path])
            prefixes_with_text[prefix_str] = cluster_for_prefix
        return prefixes_with_text

    """
        Given the rows we extract, separate them into clusters where you have overlapping rows or not.
        This is the first step to finding interleaving...

        Once we find the interleaving, we merge them in (via common parts of the paths), and create
        the lists.

        From that, we make markup and that's what we give back

        Note: we need the page_manager only to find the end token of the last row's Row HTML
    """
    def creat_row_markup(self, row_json, all_page_tokens, page_manager):
        markup = {}

        earliest_latest_row_locations = {}

        for path in row_json:  # the path defines the row...
            earliest = -1
            latest = -1

            for i in range(len(row_json[path]['rows'])):
                row = row_json[path]['rows'][i]
                loc = row['starting_token_location']
                if earliest == -1:  # first run through
                    earliest = loc
                    latest = loc
                    continue
                if loc < earliest:
                    earliest = loc
                if loc > latest:
                    latest = loc
            earliest_latest_row_locations[path] = (earliest, latest)

        overlaps = []
        for pth in earliest_latest_row_locations:
            begin = earliest_latest_row_locations[pth][0]
            end = earliest_latest_row_locations[pth][1]
            if begin == -1 or end == -1:  # ill defined locations
                continue
            if len(overlaps) == 0:  # first guy...
                overlaps.append([pth])
                continue

            overlap_clust = -1
            for clust_id in range(len(overlaps)):
                cluster = overlaps[clust_id]
                for cpath in cluster:  # could probably just find min and max of cluster and check w/ that, but easier for now...
                    p_begin = earliest_latest_row_locations[cpath][0]
                    p_end = earliest_latest_row_locations[cpath][1]

                    #  now, see if there is not  overlap...
                    if p_end < begin or p_begin > end:
                        continue
                    overlap_clust = clust_id

            if overlap_clust == -1:
                overlaps.append([pth])
            else:
                overlaps[overlap_clust].append(pth)

        table_paths = []

        for clust in overlaps:
            if self.__DEBUG:
                print "===oo00 CLUSTER 00oo==="
                print clust
            path_for_start = ""
            #left most, largest row is the beginning, so use that one as A's'
            row_start_location = 99999999999

            # first, find the member with the most rows
            max_rows = max([len(row_json[member]['rows']) for member in clust])

            # Ok, so the HTML between rows could have been messed up before bc we didn't know that these were
            # overlapping lists. For instance, the first row could be alone and now it's merged, so let's remake
            # the html between...

            for member in clust:
                num_rows = len(row_json[member]['rows'])
                if self.__DEBUG:
                    print "\t--> (%d, %d): %d" % (earliest_latest_row_locations[member][0], earliest_latest_row_locations[member][1], num_rows)
                    print "\t\t PATH: "+member
                    print '\n'.join(["\t\t\t"+str(b['starting_token_location'])+" "+b['visible_text']+": "+b['html_between_row'] for b in row_json[member]['rows']])
                if num_rows == max_rows:
                    if earliest_latest_row_locations[member][0] < row_start_location:
                        row_start_location = earliest_latest_row_locations[member][0]
                        path_for_start = member
            if self.__DEBUG:
                print ">> Row starts at: %d (%s) " % (row_start_location, path_for_start)
            table_paths.append(path_for_start)

        if self.__DEBUG:
            print '== TABLE PATHS =='
            print '\n'.join(table_paths)
        # for each table path, we need to sort the members, and then assign their inner HTML values. Note that
        # these might be empty (for first row, etc.) in which case we fill it in. But if it's there, then keep it...
        # so we turn each table path into a little regex, and starting from each token, find the next one, and use the
        # stuff between as the
        # they also need to be sorted bc we need to assign teh correct number to each
        for table_path in table_paths:
            # make the structure that we want...
            by_location = {}  # makes it easy to sort by location, etc.
            for row in row_json[table_path]['rows']:
                by_location[row['starting_token_location']] = row

            if len(by_location) < 2:
                continue

            ordered_row_indexes = sorted(by_location.keys())
            extract_sequences = []
            ending_row_locations = []  # the token location for the end of each row...

            table_path_regex = '+?'.join([tp for tp in table_path])

            # Three cases for what your extracted value could be:
            # 1 - Normal case: it's the html_between_row value
            # 2 - You are a first or optional row, so your html_between_row is empty (bc you might have been
            # on a path by yourself). So, we find it as the html between you and the next guy in this combined list
            # 3 - The last row. For this, we guess what the end looks like by looking at all of the HTML tags
            # for the html_between_row for the guy preceding it, and then find those tags from the start of the
            # last row, to the end of the HTML page
            for idx in range(len(ordered_row_indexes)):
                ordered_row_idx = ordered_row_indexes[idx]
                ext_seq = ''

                if by_location[ordered_row_idx]['html_between_row'] == '' and idx < len(ordered_row_indexes) - 1:
                    # can get the HTML as the text between this guy and the next
                    next_start_token = ordered_row_indexes[idx+1] - 1
                    sub_page = all_page_tokens.getTokensAsString(ordered_row_idx, next_start_token,
                                                                 whitespace=True)
                    ext_seq = sub_page
                else:
                    ext_seq = by_location[ordered_row_idx]['html_between_row']

                if idx < len(ordered_row_indexes) - 1:  # We don't know where the last guy ends, so we don't have this.
                    extract_sequences.append(ext_seq)
                    ending_row_locations.append(ordered_row_indexes[idx+1] - 1)

                if idx == len(ordered_row_indexes) - 1:  # last guy, so use the end_it regex and find from this guy
                    # initially was doing longest common substring for all prev rows, but you really just need
                    # # the last one, I think. Otherwise if you are mixing in optional/first-row you get weirdness...
                    found_end_loc = self.slot_to_end_token_loc(''.join(extract_sequences[-1]), all_page_tokens,
                                                               ordered_row_idx,
                                                               page_manager)

                    seen_etags = [s for s in re.findall("<[a-z]+", ''.join(extract_sequences[-1]))]

                    # now, jump to the next HTML token we see, after the occurrence of these guys...
                    rest_of_page = all_page_tokens.getTokensAsString(ordered_row_idx, len(all_page_tokens) - 1,
                                                                     whitespace=True)
                    found_match = re.search('.+?'.join(seen_etags), rest_of_page)
                    if found_match:
                        found = found_match.end()
                    else:
                        found = len(all_page_tokens) - 1

                    # now, find the next HTML tag from this point, and add that into the extract
                    # TODO: get this last token in there...

                    slot = rest_of_page[0:found]
                    extract_sequences.append(slot)
                    ending_row_locations.append(found_end_loc)

            # now, add this markup in
            markup[table_path] = {'sequence': []}
            for i in range(len(extract_sequences)):
                extract = extract_sequences[i]
                seq_number = i+1
                #start_tok_loc = by_location[ordered_row_indexes[i]]['starting_token_location']
                start_tok_loc = self.slot_to_start_loc(table_path, extract, page_manager)
                end_tok_loc = ending_row_locations[i]
                if start_tok_loc and end_tok_loc:
                    markup_value = all_page_tokens.getTokensAsString(start_tok_loc, end_tok_loc, whitespace=True)
                    markup[table_path]['sequence'].append({'extract': markup_value, 'sequence_number': seq_number,
                                                           'starting_token_location': start_tok_loc,
                                                           'ending_token_location': end_tok_loc})

        return markup

    # TODO: This could have errors bc of reliance on regex
    def slot_to_start_loc(self, rule, row_html, page_manager):
        rule_regex = rule.replace("><", ">.*?<")
#         print "ROW: %s" % row_html
#         print "RULE: %s" % rule_regex
        found_match = re.search(rule_regex, row_html)
        if found_match:
            found = found_match.end()
            possible_locs = page_manager.getPossibleLocations(page_manager.getPageIds()[0], row_html[found:])
            best_loc = possible_locs[0]  # now we've turned this slot into a location
            return best_loc[0]
        return None

    # TODO: This could have errors... lots of regex stuff...
    def slot_to_end_token_loc(self, extraction, all_page_tokens, starting_token_location, page_manager):
        seen_etags = [s for s in re.findall("<[a-z]+", extraction)]

        # now, jump to the next HTML token we see, after the occurrence of these guys...
        rest_of_page = all_page_tokens.getTokensAsString(starting_token_location, len(all_page_tokens) - 1,
                                                         whitespace=True)
        found_match = re.search('.*?'.join(seen_etags), rest_of_page)
        if found_match:
            found = found_match.end()
        else:
            return None

        # now, find the next HTML tag from this point, and add that into the extract
        # TODO: get this last token in there...

        slot = rest_of_page[0:found]

        # we know this is the slot for the only page in the page manager passed in...
        possible_locs = page_manager.getPossibleLocations(page_manager.getPageIds()[0], slot)
        best_loc = possible_locs[0]  # now we've turned this slot into a location

        if best_loc[0] == starting_token_location:
            return best_loc[-1]
        else:
            raise Exception("Could not locate the correct end token")

    """
    @param pages: A hash where key is hte page name, and value is the raw page content
    """
    def learn_list_extractors(self, pages):
        page_mgr = PageManager() #write_debug_files=True)

        markup = {}
        for page in pages:
            page_content = pages[page]
            page_mgr.addPage(page, page_content)
            content_list_markup = self.lists_on_single_page(page_content)
            markup[page] = content_list_markup

        print '--- MARKUP ---'
        print json.dumps(markup)
        page_mgr.learnStripes(markups=markup)
        rules = page_mgr.learnRulesFromMarkup(markup)


        # now, for each markup rule, learn a little page manager
        sublist_page_managers = {}
        for page in markup:
            for rule_name in markup[page]:
                if rule_name not in sublist_page_managers:
                    sublist_page_managers[rule_name] = PageManager()
                for rid in range(len(markup[page][rule_name]['sequence'])):
                    row = markup[page][rule_name]['sequence'][rid]
                    sublist_page_managers[rule_name].addPage(page+"html%d" % rid, row['extract'])

        sublist_sub_rules = {}
        for sublist in sublist_page_managers:
            sublist_page_managers[sublist].learnStripes()
            sub_rules = sublist_page_managers[sublist].learnAllRules(in_list = True)
            sublist_sub_rules[sublist] = sub_rules  # This should match a rule name in the rules...

        count = 1 
        for rule in rules.rules:
            print "== RULE INFO =="
            print str(rule.name)
            rule.set_sub_rules(sublist_sub_rules[rule.name])
            list_name = '_div_list'+format(count, '04')
            for page_id in markup:
                if rule.name in markup[page_id]:
                    markup[page_id][list_name] = markup[page_id].pop(rule.name)
            rule.name = list_name
            print str(json.dumps(rule.toJson()))
            print "==============="
#             
#         print rules.toJson()
        
        
        return rules, markup

    def lists_on_single_page(self, content):
        pg = PageManager()
        pg.addPage("zzz", content)

        triples = pg.getVisibleTokenStructure()
        (ptree, paths_to_vis_text, path_to_invis_toks) = self.prefix_tree(triples, only_consider_tag='div')

        potential_lists = self.prefix_tree_to_paths(ptree)

        if self.__DEBUG:
            print '.... POTENTIAL LISTS ARE ....'
            print '\n'.join([''.join(p) for p in potential_lists])
            print '.... OK!....'

        all_tokens_list = pg.getPage("zzz").tokens

        # Now, let's get our lists
        lists = {}

        for i in range(len(potential_lists)):
            pot_list = potential_lists[i]

            as_path = ''.join(pot_list)
            if self.__DEBUG:
                print "PATH: %s" % as_path
            lists[as_path] = {
                'rows': []
            }

            # if as_path in paths_to_vis_text:
            for path_to_vis in paths_to_vis_text:
                if path_to_vis.find(as_path) > -1:
                    vis_texts = [a for a in paths_to_vis_text[path_to_vis]]
                    invis_toks = [t for t in path_to_invis_toks[path_to_vis]]

                    for idx in range(len(vis_texts)):
                        if self.__DEBUG:
                            print "%s ==> %s" % (vis_texts[idx], str(invis_toks[idx].token_location))
                        html_between_row = ''
                        if (idx+1) < len(vis_texts):
                            begin = invis_toks[idx].token_location
                            end = invis_toks[idx+1].token_location - 1
                            html_between_row = all_tokens_list.getTokensAsString(begin, end, whitespace=True)
                        lists[as_path]['rows'].append({
                            'visible_text': vis_texts[idx],
                            'starting_token_location': invis_toks[idx].token_location,
                            'html_between_row': html_between_row
                        })
            as_json_str = json.dumps(lists)

            if self.__DEBUG:
                print "--------"
                print as_json_str
                print "--------"

            # # do it as an extraction instead?
            # item_rule_begin = Landmark.escape_regex_string('<html')
            # item_rule_end = Landmark.escape_regex_string('/html>')
            #
            # begin_iter_rule = '.+?'.join([Landmark.escape_regex_string(a) for a in pot_list])
            #
            # # figure out: for each tag in the rule, add it's end tag (keep track of tag type)
            # #  NOTE: for now, this assumes that the HTML is well formed
            # end_it = '.+?'.join(['</div>' for i in range(len(pot_list))])
            #
            # end_iter_rule = end_it
            #
            # #  include end-regex: included in the stuff that's extracted.
            # #  Solve for the case where you only see part of the stuff
            # rule = IterationRule(str(i) + "_pathListRule", item_rule_begin, item_rule_end,
            #                      begin_iter_rule, end_iter_rule, removehtml=True)
            # extraction = rule.apply(content)
            #
            # print "**PATH: "+''.join(pot_list)
            # as_json_str = json.dumps(extraction)
            #
            # for seq in extraction['sequence']:
            #     print "\t"+seq['extract']

        # TODO: do this here????
        # TODO: big drop down the path should be considered... not just if hte path occurs twice
        # TODO: fix bugs
        markup = self.creat_row_markup(lists, all_tokens_list, pg)
        if self.__DEBUG:
            print "list markup"
            json.dumps(markup)
        return markup



import os
import codecs
from landmark_ml.learning.PageManager import PageManager
import json

import logging
# logger = logging.getLogger("landmark")
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("landmark")
handler = logging.FileHandler('landmark.log')
handler.setLevel(logging.INFO)
formatter = logging.Formatter(u'%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

def main():
    #should be taken as a arg
    page_file_dir = ''
    
    pageManager = PageManager()
    files = [f for f in os.listdir(page_file_dir) if os.path.isfile(os.path.join(page_file_dir, f))]
    for the_file in files:
        if the_file.startswith('.'):
            continue
        
        with codecs.open(os.path.join(page_file_dir, the_file), "r", "utf-8") as myfile:
            page_str = myfile.read().encode('utf-8')
        
        pageManager.addPage(the_file, page_str)
    
    pageManager.learnStripes()
    (list_markup, list_names) = pageManager.learnListMarkups()
    
    rule_set = pageManager.learnAllRules()
    (markup, names) = pageManager.rulesToMarkup(rule_set)
    
    for key in markup.keys():
        if key in list_markup:
            markup[key].update(list_markup[key])

#     print json.dumps(markup, sort_keys=True, indent=2, separators=(',', ': '))
    
    rule_set = pageManager.learnRulesFromMarkup(list_markup)
    print json.dumps(json.loads(rule_set.toJson()), sort_keys=True, indent=2, separators=(',', ': '))

if __name__ == '__main__':
    main()


# This Python file uses the following encoding: utf-8

import codecs
import sys
import getopt

import logging
from landmark_ml.extraction.ExtractionCheck import run_extraction_check

import json
from landmark_extractor.extraction.Landmark import RuleSet
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("landmark")
handler = logging.FileHandler('landmark.log')
handler.setLevel(logging.INFO)
formatter = logging.Formatter(u'%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "dh", ["debug", "help"])
            
            for opt in opts:
                if opt in [('-h', ''), ('--help', '')]:
                    raise Usage('python -m learning.RuleLearnerAllSlots [OPTIONAL_PARAMS] [TEST_FILES_FOLDER] \n\t[OPTIONAL_PARAMS]: -d to get debug stripe html files')
        except getopt.error, msg:
            raise Usage(msg)
        
        logger.info('Running RuleEval with files at %s and rules %s', args[0], args[1])
        
        #read the directory location from arg0
        page_file_dir = args[0]
        
        json_file_str = args[1]
        with codecs.open(json_file_str, "r", "utf-8") as myfile:
            json_str = myfile.read().encode('utf-8')
        json_object = json.loads(json_str)
        rules = RuleSet(json_object)
        
        run_extraction_check(page_file_dir, '../../output/watable/', rules)
        
        
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2
    
if __name__ == '__main__':
    sys.exit(main())


import sys
import getopt
from landmark_ml.learning.TruffleShuffle import TruffleShuffle
import os
from shutil import copyfile
import codecs
import shutil
import json
from landmark_ml.learning.PageManager import PageManager

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def cluster(project_name, working_dir_str, copy_to_webapp=False, tokens_only=False):
    
    #try to get the right directory to get the landmark online tools folder
    if copy_to_webapp:
        working = os.getcwd()
        while not working.endswith('/src'):
            (working,other) = os.path.split(working)
        web_app_projects_dir = os.path.join(working, 'angular_flask/static/project_folders')
    
    page_manager = PageManager(page_dir=working_dir_str)
    
    tf = TruffleShuffle(page_manager)
    clusters = tf.do_truffle_shuffle(algorithm='rule_size', tokens_only=tokens_only)
    clusterCount = 1
    clusters_dir_str = os.path.join(working_dir_str, '../clusters')
    if len(clusters) > 0:
        for rule in sorted(clusters, key=lambda x: len(clusters[x]['MEMBERS']), reverse=True):
            cluster_str = 'cluster' + format(clusterCount, '03')
            clusterCount += 1
                    
            page_count = 0;
            print cluster_str + " -- " + str(len(clusters[rule]['MEMBERS']))
            if len(clusters[rule]['MEMBERS']) > 0:
                
                #copy it into the local angular_flask web directory
                markup_file = None
                if copy_to_webapp and clusterCount <= 11:
                    blank = os.path.join(web_app_projects_dir, '_blank')
                    project_dir = os.path.join(web_app_projects_dir, project_name+"_"+cluster_str)
                    shutil.copytree(blank, project_dir)
                
                    markup_file = os.path.join(project_dir, 'learning', 'markup.json')
                    with codecs.open(markup_file, "r", "utf-8") as myfile:
                        json_str = myfile.read().encode('utf-8')
                    markup = json.loads(json_str)
                
                cluster_dir_str = os.path.join(clusters_dir_str, cluster_str)
                if not os.path.exists(cluster_dir_str):
                    os.makedirs(cluster_dir_str)
                    
                for page_id in clusters[rule]['MEMBERS']:
                    copyfile(os.path.join(working_dir_str, page_id), os.path.join(cluster_dir_str, page_id))
                    
                    if copy_to_webapp and clusterCount <= 11:
                        if page_count < 7:
                            #and copy it to the web_app_dir if we have less than 7 there
                            copyfile(os.path.join(working_dir_str, page_id), os.path.join(project_dir, page_id))
                            markup['__URLS__'][page_id] = page_id
                            markup[page_id] = {}
    
                    page_count += 1
                
                with codecs.open(os.path.join(cluster_dir_str, 'cluster_meta_data.json'), "w", "utf-8") as myfile:
                    cluster_meta_data = {}
                    cluster_meta_data['name'] = cluster_str
                    cluster_meta_data['members'] = clusters[rule]['MEMBERS']
                    cluster_meta_data['_page_count'] = len(clusters[rule]['MEMBERS'])
                    cluster_meta_data['_anchor'] = clusters[rule]['ANCHOR']
                    cluster_meta_data['chunks'] = rule.strip().split(tf.get_chunk_separator())
                    myfile.write(json.dumps(cluster_meta_data, sort_keys=True, indent=2, separators=(',', ': ')))
                    myfile.close()
                
                if copy_to_webapp and clusterCount <= 11:
                    with codecs.open(markup_file, "w", "utf-8") as myfile:
                        myfile.write(json.dumps(markup, sort_keys=True, indent=2, separators=(',', ': ')))
                        myfile.close()
    else:
        cluster_str = 'cluster' + format(clusterCount, '03')
        
        #copy it into the local angular_flask web directory
        markup_file = None
        if copy_to_webapp:
            blank = os.path.join(web_app_projects_dir, '_blank')
            project_dir = os.path.join(web_app_projects_dir, project_name+"_"+cluster_str)
            shutil.copytree(blank, project_dir)
        
            markup_file = os.path.join(project_dir, 'learning', 'markup.json')
            with codecs.open(markup_file, "r", "utf-8") as myfile:
                json_str = myfile.read().encode('utf-8')
            markup = json.loads(json_str)

        clusterCount += 1
                
        page_count = 0;
        cluster_dir_str = os.path.join(clusters_dir_str, cluster_str)
        if not os.path.exists(cluster_dir_str):
            os.makedirs(cluster_dir_str)
                
        for page_id in tf.get_page_manager().getPageIds():
            copyfile(os.path.join(working_dir_str, page_id), os.path.join(cluster_dir_str, page_id))
            
            if copy_to_webapp:
                if page_count < 7:
                    #and copy it to the web_app_dir if we have less than 5 there
                    copyfile(os.path.join(working_dir_str, page_id), os.path.join(project_dir, page_id))
                    markup['__URLS__'][page_id] = page_id
                    markup[page_id] = {}

            page_count += 1
        
        if copy_to_webapp:
            with codecs.open(markup_file, "w", "utf-8") as myfile:
                myfile.write(json.dumps(markup, sort_keys=True, indent=2, separators=(',', ': ')))
                myfile.close()

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "h", ["help"])
            for opt in opts:
                if opt in [('-h', ''), ('--help', '')]:
                    raise Usage('python extraction/PageClusterer.py [WORKING_DIR]')
                
            directory = args[0]
            #now loop through each and run the clustering
            dirs = [f for f in os.listdir(directory) if os.path.isdir(os.path.join(directory, f))]
            for sub_dir in dirs:
                print '...clustering ' + sub_dir + '...'
                cluster(file, os.path.join(directory, sub_dir, 'cdr'))
            
        except getopt.error, msg:
            raise Usage(msg)
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2
    
if __name__ == "__main__":
    sys.exit(main())


# This Python file uses the following encoding: utf-8

import codecs
import sys
import getopt
import os
from landmark_ml.learning.PageManager import PageManager
import json
from landmark_extractor.extraction.Landmark import flattenResult

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "h", ["help"])
        except getopt.error, msg:
            raise Usage(msg)
        
        #read the directory location from arg0
        page_file_dir = args[0]
        
        pageManager = PageManager()
        page_str_array = []
        for subdir, dirs, files in os.walk(page_file_dir):
            for the_file in files:
                if the_file.startswith('.'):
                    continue
                
                with codecs.open(os.path.join(subdir, the_file), "r", "utf-8") as myfile:
                    page_str = myfile.read().encode('utf-8')
                    page_str_array.append(page_str)
                    
                pageManager.addPage(the_file, page_str)
                
        pageManager.learnStripes()
        
        #Read the markups from a file...
        markups_file = args[1]
        with codecs.open(os.path.join('', markups_file), "r", "utf-8") as myfile:
            markup_str = myfile.read().encode('utf-8')
        markups = json.loads(markup_str)
        
        markups.pop("__SCHEMA__", None)
        
        #Before we learn the stripes let's make sure we can open the output file
        pageManager.learnStripes(markups)
        rule_set = pageManager.learnRulesFromMarkup(markups)
        
        if(len(args) > 2):
            output_file = args[2]
            with codecs.open(output_file, "w", "utf-8") as myfile:
                myfile.write(rule_set.toJson())
                myfile.close()
        
        #testing
        flatten = False
        extraction_list = rule_set.extract(page_str_array[0])
        
        if rule_set.validate(extraction_list):
            if flatten:
                print json.dumps(flattenResult(extraction_list), sort_keys=True, indent=2, separators=(',', ': '))
            else:
                print json.dumps(extraction_list, sort_keys=True, indent=2, separators=(',', ': '))

        
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2
    
if __name__ == '__main__':
    sys.exit(main())



# ls -1 | python -c "import sys; import random; print(''.join(random.sample(sys.stdin.readlines(), int(sys.argv[1]))).rstrip())" 200 | xargs -I {} mv {} ../cdr/

import sys
import json
import getopt
import os
import codecs
import random
import shutil

crawl_file = "crawl_file"

usage = "undercrawler_to_json_files -f " + crawl_file

try:
    opts, args = getopt.getopt(sys.argv[1:], "hf:")
except getopt.GetoptError:
    print usage
    sys.exit(2)
for opt, arg in opts:
    if opt == '-h':
        print usage
        sys.exit()
    elif opt in "-f":
        crawl_file = arg

if crawl_file == 'crawl_file':
    print usage
    sys.exit(2)
    
crawl_dir = os.path.join(os.path.dirname(crawl_file), "all")
if not os.path.exists(crawl_dir):
    os.makedirs(crawl_dir)

crawl_json = []
file_names = []
with codecs.open(crawl_file, "r", "utf-8") as myfile:
    for line in myfile:
        crawl_page = json.loads(line)
        if not crawl_page['content_type'].startswith('image'):
            json_out = {}
            try:
                if 'raw_content' in crawl_page:
                    json_out['_id'] = crawl_page['_id']
                    json_out['url'] = crawl_page['url']
                    json_out['raw_content'] = crawl_page['raw_content']
                    with codecs.open(os.path.join(crawl_dir, json_out['_id']+".json"), "w", "utf-8") as json_file:
                        json_file.write(json.dumps(json_out, indent=4, separators=(',', ': ')))
                    file_names.append(json_out['_id']+".json")
            except:
                print json.dumps(crawl_page)
                sys.exit()

cdr_dir = os.path.join(os.path.dirname(crawl_file), "cdr")    
if not os.path.exists(cdr_dir):
    os.makedirs(cdr_dir)

for random_file in random.sample(file_names, 200):
    shutil.copyfile(os.path.join(crawl_dir, random_file), os.path.join(cdr_dir, random_file))



from random import shuffle

class TruffleShuffle(object):

    def __init__(self, page_manager):
        self.__chunkBreakSeparator = '<BRK>'
        self.__page_manager = page_manager

    def get_chunk_separator(self):
        return self.__chunkBreakSeparator

    def get_page_manager(self):
        return self.__page_manager

    # so maybe you table randome samples of 3 pages, and induce a template
    # if you find a template that is similar (or matches) most, then that is the template for this cluster?
    # or you could do a greedy build or something (e.g., add another page and if it doesn't change, you are good)
    def sample_and_learn_template(self, cluster_members, sub_sample_size=5, iterations=10):
        stripes = []
        for itr in range(iterations):
            shuffle(cluster_members) # randomly orders them
            random_members = cluster_members[0:sub_sample_size] # get the sub-sample
            template = self.induce_template(random_members).getStripes()

            stripe_texts = []
            for stripe in template:
                stripe_text = stripe['stripe']
                stripe_texts.append(stripe_text)

            # now, only keep the top X longest stripes and see what it does...
            top_x = 10  # for now...
            stripes_by_size = {}
            for stpe in stripe_texts:
                stsz = len(stpe)
                if stsz not in stripes_by_size:
                    stripes_by_size[stsz] = []
                stripes_by_size[stsz].append(stpe)

            top_sizes = sorted(stripes_by_size.keys(), reverse=True)
            kept_big_stripes = []
            for tsz in top_sizes:
                kept_big_stripes.extend(stripes_by_size[tsz])
                if len(kept_big_stripes) > top_x:
                    break
            # stripes_string = self.__chunkBreakSeparator.join(stripe_texts)
            stripes_string = self.__chunkBreakSeparator.join(kept_big_stripes[:top_x])
            stripes.append(stripes_string)

        template_occurrences = {}
        for tstr in stripes:
            template_occurrences[tstr] = stripes.count(tstr)

        for sstring in template_occurrences:
            if template_occurrences[sstring] > 1:
                print "Template: %s" % sstring[:250]  # just a little bit
                print "Induced template occurs %d out of %d" % (template_occurrences[sstring], iterations)

    def induce_template(self, cluster_members):
        return self.__page_manager.getSubPageManager(cluster_members)

    def prep_truffles_to_shuffle(self, tokens_only = False):
        all_chunks = set()
        page_chunks_map = {}
        for page_id in self.__page_manager.getPageIds():
            if tokens_only:
                page_chunks = set()
                for token in self.__page_manager.getPage(page_id).tokens:
                    page_chunks.add(token.token)
            else:
                page_chunks = self.__page_manager.getPageChunks(page_id)
            all_chunks.update(page_chunks)
            page_chunks_map[page_id] = page_chunks

        chunks_to_remove = set()
        all_pages_sz = len(self.__page_manager.getPageIds())
        for chunk in all_chunks:
            num_pages_with_chunk = 0
            for page_id in self.__page_manager.getPageIds():
                if chunk in page_chunks_map[page_id]:
                    num_pages_with_chunk += 1
            if tokens_only:
                if num_pages_with_chunk < 2 or num_pages_with_chunk == all_pages_sz:
                    chunks_to_remove.add(chunk)
            else:
                if num_pages_with_chunk < 10 or num_pages_with_chunk == all_pages_sz:
                    chunks_to_remove.add(chunk)

#         print str(len(all_chunks)) + " chunks before filtering"
        all_chunks.difference_update(chunks_to_remove)
        for page_id in self.__page_manager.getPageIds():
            page_chunks_map[page_id].difference_update(chunks_to_remove)

#         print str(len(all_chunks)) + " chunks left after filtering"
#         print str(all_pages_sz) + " pages total"
        return all_chunks, page_chunks_map

    def __test_good_merge(self, potential_cluster_template, test_cluster_template):
        good_merge = False
        (old_visible_count, old_invisible_count) = potential_cluster_template.countTokenInfoInStripes()
        old_clean_slot_count = potential_cluster_template.countCleanSlots()
        print '\t\told_visible = ' + str(old_visible_count)
        print '\t\told_invisible = ' + str(old_invisible_count)
        print '\t\told_clean_slot_count = ' + str(old_clean_slot_count)
        
        (new_visible_count, new_invisible_count) = test_cluster_template.countTokenInfoInStripes()
        new_clean_slot_count = test_cluster_template.countCleanSlots()
        print '\t\tnew_visible = ' + str(new_visible_count)
        print '\t\tnew_invisible = ' + str(new_invisible_count)
        print '\t\tnew_clean_slot_count = ' + str(new_clean_slot_count)
        
        if old_invisible_count == new_invisible_count and new_visible_count <= old_visible_count:
            good_merge = True
        if new_clean_slot_count >= old_clean_slot_count:
            good_merge = True
        
        return good_merge
    
#     def __test_good_merge_template_string(self, potential_cluster_template_string, test_cluster_template_string):
#         good_merge = False
#         test_template = PageManager()
#         test_template.addPage('potential', potential_cluster_template_string, add_special_tokens=False)
#         test_template.addPage('test', test_cluster_template_string, add_special_tokens=False)
#         test_template.learnStripes()
#         print len(test_template._stripes)
#         if len(test_template._stripes) < 2:
#             good_merge = True
#         
#         return good_merge, ' '.join(value['stripe'] for value in test_template._stripes)
# 
#     def __merge_clusters_template_string(self, final_clusters):
#         merged_clusters = {}
#         for final_cluster_rule in sorted(final_clusters, key=lambda x: final_clusters[x]['NAME']):
#             final_cluster_name = final_clusters[final_cluster_rule]['NAME']
#             print 'testing ' + final_cluster_name + " ..."
#             c_i = final_clusters[final_cluster_rule]
#             did_merge = False
#             for rule in merged_clusters:
#                 cluster_name = merged_clusters[rule]['NAME']
#                 print '\tgenerating merged template with ' + cluster_name + " ..."
#                 merged = merged_clusters[rule]
#                 good_merge, merged_template_string = self.__test_good_merge_template_string(c_i['TEMPLATE_STRING'], merged['TEMPLATE_STRING'])
#                 if good_merge:
#                     merged_clusters[rule]['MEMBERS'].extend(c_i['MEMBERS'])
#                     merged_clusters[rule]['TEMPLATE_STRING'] = merged_template_string
#                     did_merge = True
#                     print 'merging ' + final_cluster_name
#                     print '\tinto ' + cluster_name
#                     print '\tnew_template = ' + merged_template_string
#                     break
#                 
#             if not did_merge:
#                 merged_clusters[final_cluster_rule] = c_i
#             
#         return merged_clusters
                
    def __merge_clusters(self, final_clusters):
        merged_clusters = {}
        merged_clusters_pms = {}
        for final_cluster_rule in sorted(final_clusters, key=lambda x: final_clusters[x]['NAME']):
            final_cluster_name = final_clusters[final_cluster_rule]['NAME']
            print 'testing ' + final_cluster_name + " ..."
            c_i = final_clusters[final_cluster_rule]
            if final_cluster_rule in merged_clusters_pms:
                c_i_template = merged_clusters_pms[final_cluster_rule]
            else:
                c_i_template = self.induce_template(c_i['MEMBERS'][:5])
            
            did_merge = False
            for rule in merged_clusters:
                cluster_name = merged_clusters[rule]['NAME']
                print '\tgenerating merged template with ' + cluster_name + " ..."
                merged = merged_clusters[rule]
                
                new_members = c_i['MEMBERS'][:5]
                new_members.extend(merged['MEMBERS'][:5])
                merged_cluster_template = self.induce_template(new_members)
                
                if self.__test_good_merge(c_i_template, merged_cluster_template):
                    ###add c_i to merged - really just adding the pages to it for now
                    merged_clusters[rule]['MEMBERS'].extend(c_i['MEMBERS'])
                    merged_clusters_pms[rule] = merged_cluster_template
                    
                    did_merge = True
                    print 'merging ' + final_cluster_name
                    print '\tinto ' + cluster_name
                    break
                    
            if not did_merge:
                merged_clusters[final_cluster_rule] = c_i
                merged_clusters_pms[final_cluster_rule] = c_i_template
            
        return merged_clusters

    ##############################
    #
    # Clusters pages according to "rules". A "rule" is a list of chunks, and a "chunk" is a section of a Web page
    # that is visible to a user.
    #
    # Inputs:
    #   algorithm: 'rule_size': cluster by the size of rule from long rules to short rules
    #               'coverage' : cluster by the number of pages covered by a rule, small to big (more specific to less)
    #
    # Outputs:
    #   dict[rule] = {
    #       'MEMBERS': list of page ids (Pids from the PageManager),
    #       'ANCHOR': the anchoring chunk for this cluster
    #    }
    #   That is, each entry is a rule and its value is a dict. Note that an anchor is unique
    #   Each rule is a string of chunk_1<BRK>chunk_2<BRK>...<BRK>chunk_N
    #   it's a string to make it an index, but to use it you could break on <BRK>
    #  which you can get from the method get_chunk_separator()
    #
    ##############################
    def do_truffle_shuffle(self, algorithm='coverage', tokens_only=False):
        all_chunks, page_chunks_map = self.prep_truffles_to_shuffle(tokens_only)
        chunk_counts = {}
        seen_rules = []
        rule_anchors = {}
        for chunk in all_chunks:
            pages_with_chunk = []
            for page_id in self.__page_manager.getPageIds():
                if chunk in page_chunks_map[page_id]:
                    pages_with_chunk.append(page_id)
            other_chunks = set()
            other_chunks.update(page_chunks_map[pages_with_chunk[0]])
            for page_id in pages_with_chunk:
                other_chunks.intersection_update(page_chunks_map[page_id])

            # now, find all the guys that have all of those chunks...
            if len(other_chunks) > 1: # one token is not enough, enforce that there are at least 2...
                rule = self.__chunkBreakSeparator.join(other_chunks)
                if rule not in seen_rules:
                    chunk_counts[rule] = pages_with_chunk
                    rule_anchors[rule] = chunk
                    
        if algorithm == 'coverage':
            counts = dict([(rule, len(chunk_counts[rule])) for rule in chunk_counts])
        else:
            # count by the size of the rule, but prefer longer,
            # so make it negative so we don't need to change sorted() call below (e.g., make rules negative
            # so that sorted small to large actually gives us longer rules (more negative) to shorter (less neg)
            counts = dict([(rule, -len(rule.split(self.__chunkBreakSeparator))) for rule in chunk_counts])

        inverted = {}
        for rl in counts:
            sz = counts[rl]
            if sz not in inverted:
                inverted[sz] = []
            inverted[sz].append(rl)
        final_clusters = {}
        already_clustered = []
        for size in sorted(inverted.keys()):
            rules = inverted[size]
            for rule in rules:
                pids = [p for p in chunk_counts[rule] if p not in already_clustered]
                if len(pids) > 1:
                    already_clustered.extend(pids)
                    final_clusters[rule] = {
                        'MEMBERS': pids,
                        'ANCHOR': rule_anchors[rule]
                    }
        
#         clusterCount = 1
#         for rule in sorted(final_clusters, key=lambda x: len(final_clusters[x]['MEMBERS']), reverse=True):
#             
#             final_clusters[rule]['NAME'] = 'cluster' + format(clusterCount, '03')
#             template = self.induce_template(final_clusters[rule]['MEMBERS'])
#             template.learnStripes()
#             final_clusters[rule]['TEMPLATE_STRING'] = ' '.join(value['stripe'] for value in template._stripes)
#             clusterCount += 1
# 
#         return self.__merge_clusters(final_clusters)
        return final_clusters

# look at the coverage of the strips (# of chars that show up in all slots vs all stripes). The stripes should cover a lot of the page
# huge slots = bad sign
# invisible stuff should be mostly in the stripes. Other than visible stuff in page, how much of rest is covered in a stripe?

# how can we predict if a cluster is homogenous?


import sys
import json
import getopt
import os
import codecs
import webbrowser
import shutil

from landmark_extractor.extraction.Landmark import RuleSet
from extraction.ExtractionCheck import run_extraction_check

rules_file_string = 'step03_rules.json'
output_file_location = '_output_TEMPLATE'
directory = "directory"
usage = "step_03b_tld_rules_to_extractions -d " + directory

def single_site_run(sub_dir):
    print '...applying rules to ' + sub_dir + '...'
    cdr_dir = os.path.join(directory, sub_dir, 'cdr')
    with codecs.open(os.path.join(directory, sub_dir, rules_file_string), "r", "utf-8") as myfile:
        json_str = myfile.read().encode('utf-8')
    json_object = json.loads(json_str)
    rules = RuleSet(json_object)
    
    output_template_dir = os.path.join(os.getcwd(), output_file_location)
    output_dir = os.path.join(directory, sub_dir, '_extraction_eval')
    if os.path.exists(output_dir):
        shutil.rmtree(output_dir)
    shutil.copytree(output_template_dir, output_dir)
    
    run_extraction_check(cdr_dir, os.path.join(output_dir, 'watable'), rules, is_cdr_format = True)
    webbrowser.open_new('file://'+ os.path.join(output_dir, 'watable', 'eval.html') )

try:
    opts, args = getopt.getopt(sys.argv[1:], "hd:")
except getopt.GetoptError:
    print usage
    sys.exit(2)
for opt, arg in opts:
    if opt == '-h':
        print usage
        sys.exit()
    elif opt in "-d":
        directory = arg

single_site = ''
if len(args) > 0:
    single_site = args[0]

if directory == 'directory':
    print usage
    sys.exit(2)

if single_site:
    single_site_run(single_site)
else:
    dirs = [f for f in os.listdir(directory) if os.path.isdir(os.path.join(directory, f))]
    for sub_dir in dirs:
        single_site_run(sub_dir)



import sys
import codecs
import json
import getopt
import os
from landmark_ml.learning.PageManager import PageManager

def main(argv=None):
    if argv is None:
        argv = sys.argv
        
    opts, args = getopt.getopt(argv[1:], "h", ["help"])
    project_name = args[0]
    
    working = os.getcwd()
    while not working.endswith('/src'):
        (working,other) = os.path.split(working)
    web_app_projects_dir = os.path.join(working, 'angular_flask/static/project_folders')
    
    directory = os.path.join(web_app_projects_dir, project_name)
    markup_file = os.path.join(directory, 'learning', 'markup.json')
    with codecs.open(markup_file, "r", "utf-8") as myfile:
        json_str = myfile.read().encode('utf-8')
        
    markup = json.loads(json_str)
    
    pageManager = PageManager()
    for key in markup['__URLS__']:
        page_file = os.path.join(directory, key)
        with codecs.open(page_file, "r", "utf-8") as myfile:
            page_str = myfile.read().encode('utf-8')
        pageManager.addPage(key, page_str)

    markup.pop("__SCHEMA__", None)
    markup.pop("__URLS__", None)

    pageManager.learnStripes(markup)
    rule_set = pageManager.learnRulesFromMarkup(markup)
    
    print rule_set.toJson()

if __name__ == '__main__':
    sys.exit(main())


import sys
import getopt
import os
import codecs
import json
import shutil

def jl_to_html_and_json(json_lines_file, output_directory):
    cdr_dir = os.path.join(output_directory, "cdr")
    if os.path.exists(cdr_dir):
        shutil.rmtree(cdr_dir)
    os.makedirs(cdr_dir)
        
    html_dir = os.path.join(output_directory, "html")
    if os.path.exists(html_dir):
        shutil.rmtree(html_dir)
    os.makedirs(html_dir)
    
    with codecs.open(json_lines_file, "r", "utf-8") as jl_file:
        for line in jl_file.read().splitlines():
            if line:
                try:
                    crawl_page = json.loads(line)
                    with codecs.open(os.path.join(cdr_dir, crawl_page['_id']+'.json'), "w", "utf-8") as output_file:
                        output_file.write(line)
                        output_file.close()
                    
                    with codecs.open(os.path.join(html_dir, crawl_page['_id']+'.html'), "w", "utf-8") as output_file:
                        output_file.write(crawl_page['raw_content'])
                        output_file.close()
            
                except:
                    print ('skipping line ...')
                    print line
            else:
                break
        
class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "h", ["help"])
            for opt, arg in opts:
                if opt in [('-h', ''), ('--help', '')]:
                    raise Usage('python jl_to_html_and_json -l limit [JL_FILE]')
            
            if len(args) == 1:
                jl_file = args[0]
                dir_path = os.path.dirname(os.path.realpath(jl_file))
                jl_to_html_and_json(jl_file, dir_path)
            else:
                raise Usage('python jl_to_html_and_json -l limit [JL_FILE]')
            
        except getopt.error, msg:
            raise Usage(msg)
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2

if __name__ == "__main__":
    sys.exit(main())


# This Python file uses the following encoding: utf-8

import codecs
import sys
import getopt
import os
from landmark_ml.learning.PageManager import PageManager
import json
import time

import logging
# logger = logging.getLogger("landmark")
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("landmark")
handler = logging.FileHandler('landmark.log')
handler.setLevel(logging.INFO)
formatter = logging.Formatter(u'%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def learn_rules_with_markup(markup_file, pages_map):
    print pages_map
    for page in pages_map:
        print page

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "dh", ["debug", "help"])
            
            write_debug_files = False
            
            for opt in opts:
                if opt in [('-d', ''), ('--debug', '')]:
                    write_debug_files = True
                if opt in [('-h', ''), ('--help', '')]:
                    raise Usage('python -m learning.RuleLearner [OPTIONAL_PARAMS] [TEST_FILES_FOLDER] [MARKUP_FILE]\n\t[OPTIONAL_PARAMS]: -d to get debug stripe html files')
        except getopt.error, msg:
            raise Usage(msg)
        
        logger.info('Running RuleLearner with file at %s for rules %s', args[0], args[1])
        
        #read the directory location from arg0
        page_file_dir = args[0]
        
        pageManager = PageManager(write_debug_files)
        
        start_time = time.time()
        for subdir, dirs, files in os.walk(page_file_dir):
            for the_file in files:
                if the_file.startswith('.'):
                    continue
                
                with codecs.open(os.path.join(subdir, the_file), "r", "utf-8") as myfile:
                    page_str = myfile.read().encode('utf-8')
                    
                pageManager.addPage(the_file, page_str)
        logger.info("--- LOAD PAGES: %s seconds ---" % (time.time() - start_time))
        
        #Read the markups from a file...
        start_time = time.time()
        markups_file = args[1]
        with codecs.open(markups_file, "r", "utf-8") as myfile:
            markup_str = myfile.read().encode('utf-8')
        markups = json.loads(markup_str)
        
        markups.pop("__SCHEMA__", None)
        markups.pop("__URLS__", None)
        
        logger.info("--- LOAD MARKUPS: %s seconds ---" % (time.time() - start_time))

        pageManager.learnStripes(markups)
        start_time = time.time()
        rule_set = pageManager.learnRulesFromMarkup(markups)
        logger.info("--- LEARN RULES FROM MARKUP: %s seconds ---" % (time.time() - start_time))
         
        if(len(args) > 2):
            output_file = args[2]
            with codecs.open(output_file, "w", "utf-8") as myfile:
                myfile.write(rule_set.toJson())
                myfile.close()
        else:
            print rule_set.toJson()
        
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2
    
if __name__ == '__main__':
    start_time = time.time()
    main()
    logger.info("--- %s seconds ---" % (time.time() - start_time))
#     sys.exit(main())


import sys
import getopt
from landmark_ml.learning.PageManager import PageManager
import codecs
import os
import re

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:
            opts, args = getopt.getopt(argv[1:], "h", ["help"])
        except getopt.error, msg:
            raise Usage(msg)
        
#         pageManager = PageManager()
#         page_file_dir = args[0]
#         
#         for subdir, dirs, files in os.walk(page_file_dir):
#             for file_pointer in files:
#                 if file_pointer.startswith('.'):
#                     continue
#                 
#                 with codecs.open(os.path.join(subdir, file_pointer), "r", "utf-8") as myfile:
#                     page_str = myfile.read().encode('utf-8')
#                     
#                 pageManager.addPage(page_str)
#         
#         pageManager.learnStripes()
#         page_markups = []
#         pageManager.cleanStripes(page_markups)
#         
#         for page_id in pageManager.getPageIds():
#             print '====PAGE ' + str(page_id) + "===="
#             count = 0
#             for slot in pageManager.getSlots(page_id):
#                 print "SLOT " + str(count) + ":" + slot['extract']
#                 count = count + 1
#             print ''

#TEST of REGEX
        test_string = "testing brad pitt is nice and so is will smith. here is the list now <li>brad pitt</li><li>will smith</li><li>kane see</li><li>steve minton</li>"
        first = 'brad pitt'
        second = 'will smith'
        last = 'steve minton'
        markup = first+'.*?'+second+'.*?'+last
        indexes = [m.start() for m in re.finditer(first, test_string)]
        for index in indexes:
            index_sets = [[m.start()+index, m.end()+index] for m in re.finditer(markup, test_string[index:])]
            for index_set in index_sets:
                print test_string[index_set[0]:index_set[1]]
        
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2

if __name__ == '__main__':
    sys.exit(main())